package com.lib.rncastlabs;

import android.app.Activity;
import android.content.Context;
import android.media.MediaCodec;
import android.os.Bundle;
import android.util.Log;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.widget.FrameLayout;
import com.castlabs.android.SdkConsts;
import com.castlabs.android.adverts.Ad;
import com.castlabs.android.adverts.AdInterface;
import com.castlabs.android.drm.DrmConfiguration;
import com.castlabs.android.drm.KeyStore;
import com.castlabs.android.player.AbstractPlayerListener;
import com.castlabs.android.player.PlayerConfig;
import com.castlabs.android.player.PlayerController;
import com.castlabs.android.player.PlayerView;
import com.castlabs.android.player.SourceSelector;
import com.castlabs.android.player.exceptions.CastlabsPlayerException;
import com.facebook.react.bridge.LifecycleEventListener;
import com.facebook.react.uimanager.ThemedReactContext;

public class ReactCastLabsPlayerView extends FrameLayout implements LifecycleEventListener {

  public static final String TAG = ReactCastLabsPlayerView.class.getSimpleName();

  private final ThemedReactContext themedReactContext;
  private final VideoEventEmitter eventEmitter;
  private PlayerView playerView;
  private Bundle playbackBundle;

  public ReactCastLabsPlayerView(@NonNull ThemedReactContext context) {
    super(context);
    this.themedReactContext = context;
    this.eventEmitter = new VideoEventEmitter(context);

    createViews();
  }

  @Override
  public void setId(int id) {
    super.setId(id);
    eventEmitter.setViewId(id);
  }

  @Override protected void onAttachedToWindow() {
    super.onAttachedToWindow();
    Activity activity = themedReactContext.getCurrentActivity();
    if(activity != null) playerView.getLifecycleDelegate().start(activity);
  }

  @Override protected void onDetachedFromWindow() {
    super.onDetachedFromWindow();
    playerView.getLifecycleDelegate().releasePlayer(false);
  }

  @Override public void onHostResume() {
    playerView.getLifecycleDelegate().resume();
  }

  @Override public void onHostPause() {
   playerView.getPlayerController().pause();
  }

  @Override public void onHostDestroy() {
     playerView.getLifecycleDelegate().releasePlayer(false);
  }

  public void cleanupResources() {

  }

  public void setSrc(@NonNull Bundle bundle){
    playbackBundle = bundle;
    initPlayer();
  }

  private void createViews() {
    LayoutParams layoutParams = new LayoutParams(
        LayoutParams.MATCH_PARENT,
        LayoutParams.MATCH_PARENT);
    playerView = new PlayerView(getContext());
    playerView.setLayoutParams(layoutParams);
    addView(playerView);
  }

  private void initPlayer(){
    if (playbackBundle != null) {
      try {
        // Subscribe for the playback errors
        final DrmConfiguration drmConfiguration = (DrmConfiguration) playbackBundle.get(SdkConsts.INTENT_DRM_CONFIGURATION);

        final Bundle finalPlaybackBundle = playbackBundle;
        playerView.getPlayerController().addPlayerListener(new AbstractPlayerListener() {
          @Override
          public void onError(@NonNull CastlabsPlayerException error) {
            // Handle expired offline licenses.
            // Check if the error is either TYPE_KEY_EXPIRED or TYPE_VIDEO_DECRYPTION_ERROR with
            // the cause being MediaCodec.CryptoException.ERROR_NO_KEY. In the first
            // case the key is expired and we can safely decided to re-load and see if
            // we get an new key from the DRM system. The second case unfortunately has
            // more than one meaning. It could imply that no key was loaded for the
            // content, i.e. the _wrong_ key was used for instance due to a misalignment
            // with the offlineId in the keystore.
            // It could however _also_ mean that the user fiddled around with his device
            // time and the CDM loaded the license but refuses to use it. We can still
            // remove the key and load again, since the DRM response should
            // give us a license (or not) and we would have a definitive answer.
            if (error.getType() == CastlabsPlayerException.TYPE_KEY_EXPIRED ||
                ((error.getType() == CastlabsPlayerException.TYPE_VIDEO_DECRYPTION_ERROR ||
                    error.getType() == CastlabsPlayerException.TYPE_AUDIO_DECRYPTION_ERROR) &&
                    (error.getCause() instanceof MediaCodec.CryptoException) &&
                    ((MediaCodec.CryptoException) error.getCause()).getErrorCode() == MediaCodec.CryptoException.ERROR_NO_KEY)) {

              KeyStore keyStore = PlayerController.getKeyStore();
              if(keyStore != null && drmConfiguration != null && drmConfiguration.offlineId != null) {
                if (keyStore.delete(drmConfiguration.offlineId)){
                  Log.e(TAG, "DRM license key expired or not usable. Cleared key and reload");
                  playerView.getPlayerController().open(finalPlaybackBundle);
                  return;
                }
              }
            }


            Log.e(TAG, "Error while opening player: " + error.getMessage());
          }
        });
        // (Optional) Install the CDN fallback implementation
        playerView.getPlayerController().setSourceSelector(new SourceSelector() {
          @Nullable
          @Override
          public SourceData onError(@NonNull String manifestUrl, @NonNull Exception error) {
            // Have the fallback CDN only for one particular URL
            //if (manifestUrl.equals("https://demo.cf.castlabs.com/media/QA/QA_BBB_single_4/Manifest.mpd")) {
            //  return new SourceData("https://demo.cf.castlabs.com/media/QA/QA_BBB_single_2/Manifest.mpd");
            //}
            // No fallback CDN for other URLs
            return null;
          }

          @Nullable
          @Override
          public PlayerConfig onRestart(@NonNull String manifestUrl, @Nullable PlayerConfig playerConfig) {
            // Do the 'hard' restart for the selected URL when the 'soft' one fails
            //if (manifestUrl.equals("https://demo.cf.castlabs.com/media/QA/QA_BBB_single_4/Manifest.mpd")) {
            //  // Simplify here and just update the currently failed player config with the fallback CDN URL
            //  // Usually, ensure that the DRM and other player config parameters are valid for the
            //  // returned player config or create a completely new player config
            //  if (playerConfig != null) {
            //    return new PlayerConfig.Builder(playerConfig)
            //        .contentUrl("https://demo.cf.castlabs.com/media/QA/QA_BBB_single_2/Manifest.mpd")
            //        .get();
            //  }
            //}
            // No fallback CDN for other URLs
            return null;
          }
        });

        // (Optional) Add ad events listener
        playerView.getPlayerController().getAdInterface().addAdListener(new AdInterface.Listener() {
          @Override
          public void onAdStarted(@NonNull Ad ad) {
            Log.d(TAG, "Ad started : " + ad.id + ", position = " + ad.position);
            // May hide controls and disable touch event triggers
          }

          @Override
          public void onAdCompleted() {
            Log.d(TAG, "Ad completed");
            // May show the controls again if we keep them on screen
          }

          @Override
          public void onAdPlaybackPositionChanged(long playbackPositionMs) {
          }
        });


        // Need to pass the bundle on to the PlayerController
        // to start playback. The open() method might throw an Exception in case the bundle
        // contains not all mandatory parameters or the parameters are malformed.
        playerView.getPlayerController().open(playbackBundle);
      } catch (Exception e) {
        Log.e(TAG, "Error while opening player: " + e.getMessage(), e);
      }
    } else {
      Log.e(TAG, "Can not start playback: no bundle specified");
    }
  }
}
